#!/usr/bin/env bash

broker_pid_main=
broker_pid_backup=

## include
. bash_test_lib.sh

# logs to be backed up
BACKUP_LOG_LIST=( logs/broker-*-*.log )

CLIENT_AUTH_TEST_ENABLE=false


## TEST INTERNAL PARAMS
cluster_name="auth-cluster-$(hostname)"
auth_mech=
username=
password=

ppath=/usr/share/doc/python-qpid-0.10/examples/api
spout=${ppath}/spout
drain=${ppath}/drain

#int_mech_list=( ANONYMOUS PLAIN DIGEST-MD5 )
#username_list=( asdf   asdf  guest  guest )
#password_list=( qwerty guest qwerty guest )

## OVERRIDING$
int_mech_list=( DIGEST-MD5 DIGEST-MD5 DIGEST-MD5 DIGEST-MD5)
username_list=( guest guest guest guest)
password_list=( guest guest guest guest)


function start_main_broker {
  start_broker 2
  broker_pid_main=$!
}

function start_backup_broker {
  start_broker 3
  broker_pid_backup=$!
}

function start_broker { #1: broker ID - determining data-dir and port 
  ## broker ID should be 2 for main, artbirar for backup
  ## broker ID should be lower than 10

  if (( $# == 0 )); then
    echo "Error: internal error" 1>&2
    exit 2
  fi
  local broker_id=$1
  #local lpid=

  echo "[$auth_mech] $username:$password"

  qpidd --cluster-name=$cluster_name --auth=yes \
   --cluster-mechanism=${auth_mech} \
   --cluster-username=${username} \
   --cluster-password=${password} \
   --log-enable=trace+ \
   --log-to-file=logs/broker-${broker_id}-${test_part}.log \
   --data-dir=/tmp/qpidd-${RANDOM}-$$-${broker_id} \
   --port=567${broker_id} > /dev/null 2>&1 &

  #lpid=$!
  #echo $lpid

}

function stop_all {
  for proc in $broker_pid_main $broker_pid_backup;
  do
    kill -0 $proc && kill $proc
  done
  
  broker_pid_main=
  broker_pid_backup=

  wait
}


function print_summary {
  echo "--- Summary ---"
  echo " ${err_count}/${test_count} failed"
  echo "---------------"

}

function status_check {
# process qpidd check
  ps -ef | grep qpidd | grep -v grep | sed 's/^/# /'
# process tree
  pstree -p $$ | sed 's/^/# /'

}

function clear_broker_logs {
  rm -f logs/broker-*-*.log
}

function backup_qpid_sasl_config {
  cp -f /etc/sasl2/qpidd.conf ./qpidd_sasl.conf.backup
}

function restore_qpid_sasl_config {
  cp -f ./qpidd_sasl.conf.backup /etc/sasl2/qpidd.conf 
}


# supplied mechanism list will be appended to default config
# and given to sasl
function enable_qpid_sasl_auth_mechanism { # @: mechanism list
  (( $# == 0 )) && echo "Internal Error, ${FUNCNAME[0]}" && exit 2
  cp -f ./default_qpidd_sasl.conf /etc/sasl2/qpidd.conf
  
  echo -e "# THIS IS TESTCASE bz730017's CONFIGURATION\n" \
          "mech_list: $@" >> /etc/sasl2/qpidd.conf

  local mech_list_result=$(grep "mech_list:" /etc/sasl2/qpidd.conf)
  echo "SASL MECHANISM: ${mech_list_result}"
  [[ -n ${mech_list_result} ]]
}

function check_for_cluster { # 1: log to check
  local log_result=$(grep -c "notice Running in a cluster" "$1")
  echo "####################################"
  echo "$1: $(grep --color "notice Running in a cluster" "$1")"
  echo "####################################"
  if ((  $log_result == 1 )); then
    return 0 ## means OK running in cluster
  elif (( $log_result == 0 )); then
    return 1 ## means fail not running in cluster
  else
    return 32
  fi

  ## TODO check cannot joint and cannot detect

}


function spout {
  ${spout}  $q #> /tmp/spout-$$-${test_part}
}

function drain {
  ${drain} -b 'guest:guest@localhost:5673' $q #> /tmp/drain-$$-${test_part}

}

function client_auth { #1: port
  echo "
  qpid-perftest --count 1 -s \
    --mechanism ${cauth_mech} \
    --username ${cusername} \
    --password ${cpassword} \
    --log-enable debug+ \
    -p ${1:-5672}
  "
  qpid-perftest --count 1 -s \
    --mechanism ${cauth_mech} \
    --username ${cusername} \
    --password ${cpassword} \
    --log-enable debug+ \
    -p ${1:-5672}
}

function client_send_test { # $@pass to client auth
  local cauth_mech=$auth_mech
  local cusername=guest
  local cpassword=guest
  client_auth $@
}

function all_client_auth {

  local cauth_mech=
  local cusername=
  local cpassword=
  local expected=
  local i=
  for cauth_mech in ${int_mech_list[@]} ; do
    for (( i=0; i < ${#username_list[@]}; i++ )); do
        expected=true ## bad auth mech
        [[ $cauth_mech != $auth_mech ]] && expected=false
        cusername=${username_list[$i]}
        cpassword=${password_list[$i]}
        if [[ $cauth_mech != ANONYMOUS ]]; then
          if [[ $cusername == asdf || $cpassword == qwerty ]]; then
            expected=false ## bad credentials expectation
          fi
        fi
        client_auth > /dev/null 2>&1
      countResult $? $expected "Client auth test ${cusername}/${cpassword}\
 [${cauth_mech}/${auth_mech}]"
    done

  done

}


function parse_msg_id {
  sed "s/^.*'spout-id': u\?'\([^']*\)'.*$/\1/"
}

function clients_cluster_test {
#  local q="cluster_auth-$$-${test_part}"
#  local sent=
#  local recv=


### make queue
#  qpid-config add queue $q
#
### spout message
#  sent=$(spout | parse_msg_id)
#
### drain message
#  recv=$(drain | parse_msg_id)
#
### compare
#  [[ -z ${sent} ]] && return 3
#  [[ ${sent} == ${recv} ]] 

  client_send_test && \
  client_send_test 5673

}

function auth_test_core { # 1: true/false for auth test type
    local auth_type=$1

    enable_qpid_sasl_auth_mechanism $auth_mech
    countResult $? true "Enabling SASL mechanism $auth_mech"

    start_main_broker
    sleep 3
    kill -0 $broker_pid_main
    countResult $? true "Main broker running"

    start_backup_broker
    sleep 3
    kill -0 $broker_pid_main
    countResult $? true "Backup broker running"

    if [[ $CLIENT_AUTH_TEST_ENABLE == true ]]; then
      client_send_test
      countResult $? true "Client messaging test to MAIN"

      client_send_test 5673
      countResult $? $auth_type "Client messaging test to BACKUP"

      all_client_auth ## test score is managed within
    fi

    stop_all

    sleep 5

    check_for_cluster "logs/broker-3-${test_part}.log"
    countResult $? $auth_type "Cluster connection check from BACKUP"

}

## -- main test --

makeEnvironment root

{
  test_part=0

    backup_qpid_sasl_config
    ## clear_broker_logs ## DISABLED because of backing up old logs

    beginTest "Stopping previous brokers"
      service qpidd stop &> /dev/null
      sleep 1
      killall qpidd &> /dev/null
      sleep 3
      RESULT=$(ps -ef  | grep 'qpidd' | grep -vc 'grep')
    endTest $RESULT 0

    beginTest "OpenAIS/Corosync has to be running"

      result=$(service openais status | grep -c running)
      countResult $result 1 "OpenAIS runining"
      result=$(service corosync status | grep -c running)
      countResult $result 1 "Corosync runining"

    endTest $LOCALSCORE 1

## TODO after update of bash test lib, remake to needed test
    if (( $LOCALSCORE != 1 )); then
      echo "No clustering communication service running"
      exit 2
    fi


### actual testing
    exp=
    for auth_mech in ${int_mech_list[@]} ; do
      for (( i=0; i < ${#username_list[@]}; i++ )); do
        (( test_part++ ))
        exp=true
        username=${username_list[$i]}
        password=${password_list[$i]}
        testname="Cluster auth test ${username}/${password}"
        testname+=" [${auth_mech}] #$test_part"

        beginTest "$testname"
          if [[ $auth_mech != ANONYMOUS ]]; then
            if [[ $username == asdf || $password == qwerty ]]; then
              exp=false ## bad credentials expectation
            fi
          fi
          echo "Expected: $exp"
          auth_test_core $exp

        endTest $LOCALSCORE 0
      done

    done




    restore_qpid_sasl_config

  printTestResults
} 2>&1 | tee -a ${DETAILFILE}



